from zope.component import getMultiAdapter
from ftw.permissionmanager.testing import (
    FTW_PERMISSIONMANAGER_INTEGRATION_TESTING,
    TEST_USER_ID_2, TEST_GROUP_ID, TEST_GROUP_ID_2)
import unittest2 as unittest
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD, TEST_USER_ID
from plone.testing.z2 import Browser
import transaction



class TestCopyPermissions(unittest.TestCase):

    layer = FTW_PERMISSIONMANAGER_INTEGRATION_TESTING


    def test_copy_permission_view(self):
        portal = self.layer['portal']
        view = getMultiAdapter(
            (portal.folder1, portal.folder1.REQUEST),
            name="copy_user_permissions")
        self.assertTrue(view.__name__ == 'copy_user_permissions')


    def test_copy_permission_form_user(self):
        portal = self.layer['portal']
        portal_url = portal.absolute_url()

        # Set local roles for test user
        portal.portal_membership.setLocalRoles(
            obj=portal.folder1,
            member_ids=[TEST_USER_ID],
            member_role="Contributor",
            reindex=True)
        portal.folder1.reindexObjectSecurity()

        portal.portal_membership.setLocalRoles(
            obj=portal.folder1.folder2,
            member_ids=[TEST_USER_ID],
            member_role="Editor",
            reindex=True)
        portal.folder1.folder2.reindexObjectSecurity()

        transaction.commit() # for test browser

        # Login as test user
        browser = Browser(self.layer['app'])
        browser.open(portal_url + '/login_form')
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl(name='submit').click()

        browser.open(portal_url + '/folder1/copy_user_permissions')

        # Look for the right form
        self.assertIn(
            '<form method="post" action="http://nohost/plone/folder1/@@copy_user_permissions">',
            browser.contents)

        # Search for the test user
        browser.getControl(name="search_source_user").value = TEST_USER_ID
        browser.getControl(name="submit").click()
        self.assertIn(
            '<a href="http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_user_1_">test_user_1_</a>',
            browser.contents)

        # Choose user
        browser.open('http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_user_1_')
        self.assertIn(
            '<input type="hidden" name="source_user" value="test_user_1_" />',
            browser.contents
        )

        # Choose target user
        browser.getControl(name="search_target_user").value = TEST_USER_ID_2
        browser.getControl(name="submit").click()
        self.assertIn(
            '<a href="http://nohost/plone/folder1/@@copy_user_permissions?target_user=_test_user_2_&amp;source_user=test_user_1_">_test_user_2_</a>',
            browser.contents
        )

        browser.open('http://nohost/plone/folder1/@@copy_user_permissions?target_user=_test_user_2_&amp;source_user=test_user_1_')

        # Confirm link
        self.assertIn(
            '<a class="context" href="http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_user_1_&amp;target_user=_test_user_2_&amp;confirm=1">',
            browser.contents)

        # Abort link
        self.assertIn(
            '<a class="standalone" href="http://nohost/plone/folder1/@@copy_user_permissions">',
            browser.contents)

        browser.open('http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_user_1_&amp;target_user=_test_user_2_&amp;confirm=1')

        # After permission copy, redirect to copy permission view again
        self.assertIn(
            'class="template-copy_user_permissions',
            browser.contents)

        # Approve if test user 2 has the same local roles
        self.assertTrue(
            portal.folder1.get_local_roles_for_userid(TEST_USER_ID_2) == ('Owner', 'Contributor'))
        self.assertFalse(
            portal.folder1.get_local_roles_for_userid(TEST_USER_ID_2) == ('Owner', 'Editor'))



    def test_copy_permission_form_group(self):
        portal = self.layer['portal']
        portal_url = portal.absolute_url()


        # Set up two groups
        portal.portal_groups.addGroup(TEST_GROUP_ID)
        portal.portal_groups.addGroup(TEST_GROUP_ID_2)
        # Add local roles
        portal.folder1.manage_addLocalRoles(TEST_GROUP_ID, ['Contributor',])
        portal.folder1.reindexObjectSecurity()
        portal.folder1.folder2.manage_addLocalRoles(TEST_GROUP_ID, ['Editor',])
        portal.folder1.folder2.reindexObjectSecurity()

        transaction.commit() # for test browser

        # Login as test user
        browser = Browser(self.layer['app'])
        browser.open(portal_url + '/login_form')
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl(name='submit').click()

        browser.open(portal_url + '/folder1/copy_user_permissions')

        # Look for the right form
        self.assertIn(
            '<form method="post" action="http://nohost/plone/folder1/@@copy_user_permissions">',
            browser.contents)

        # Search for the test group
        browser.getControl(name="search_source_user").value = TEST_GROUP_ID
        browser.getControl(name="submit").click()
        self.assertIn(
            '<a href="http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_group">test_group</a>',
            browser.contents)

        # Choose group
        browser.open('http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_group')
        self.assertIn(
            '<input type="hidden" name="source_user" value="test_group" />',
            browser.contents
        )

        # Choose target user
        browser.getControl(name="search_target_user").value = TEST_GROUP_ID_2
        browser.getControl(name="submit").click()
        self.assertIn(
            '<a href="http://nohost/plone/folder1/@@copy_user_permissions?target_user=test_group_2&amp;source_user=test_group">test_group_2</a>',
            browser.contents
        )

        browser.open('http://nohost/plone/folder1/@@copy_user_permissions?target_user=test_group_2&amp;source_user=test_group')

        # Confirm link
        self.assertIn(
            '<a class="context" href="http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_group&amp;target_user=test_group_2&amp;confirm=1">',
            browser.contents)

        # Abort link
        self.assertIn(
            '<a class="standalone" href="http://nohost/plone/folder1/@@copy_user_permissions">',
            browser.contents)

        browser.open('http://nohost/plone/folder1/@@copy_user_permissions?source_user=test_group&amp;target_user=test_group_2&amp;confirm=1')

        # After permission copy, redirect to copy permission view again
        self.assertIn(
            'class="template-copy_user_permissions',
            browser.contents)

        for id_, roles in dict(portal.folder1.get_local_roles()).items():
            if id_ == TEST_GROUP_ID_2:
                break
        self.assertIn('Contributor', roles)

        for id_, roles in dict(portal.folder1.folder2.get_local_roles()).items():
            if id_ == TEST_GROUP_ID_2:
                break
        self.assertIn('Editor', roles)




from zope.component import getMultiAdapter
from ftw.permissionmanager.testing import FTW_PERMISSIONMANAGER_INTEGRATION_TESTING
import unittest2 as unittest
from plone.app.testing import TEST_USER_NAME, TEST_USER_PASSWORD, TEST_USER_ID
from plone.testing.z2 import Browser
import transaction

class TestRemovePermissions(unittest.TestCase):

    layer = FTW_PERMISSIONMANAGER_INTEGRATION_TESTING


    def test_remove_permission_view(self):
        portal = self.layer['portal']
        view = getMultiAdapter(
            (portal.folder1, portal.folder1.REQUEST),
            name="remove_user_permissions")
        self.assertTrue(view.__class__.__name__ == 'RemoveUserPermissionsView')


    def test_remove_permission_form_user(self):
        portal = self.layer['portal']
        portal_url = portal.absolute_url()

        portal.portal_membership.setLocalRoles(
            obj=portal.folder1,
            member_ids=[TEST_USER_ID],
            member_role="Contributor",
            reindex=True)

        portal.portal_membership.setLocalRoles(
            obj=portal.folder1.folder2,
            member_ids=[TEST_USER_ID],
            member_role="Editor",
            reindex=True)

        browser = Browser(self.layer['app'])

        # Login as test user
        browser.open(portal_url + '/login_form')
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl(name='submit').click()

        browser.open(portal_url + '/folder1/remove_user_permissions')
        # Look for the right form
        self.assertIn(
            '<form method="post" action="http://nohost/plone/folder1/@@remove_user_permissions">',
            browser.contents
        )


        # Search for a user, there should be the test user
        browser.getControl(name="search_term").value = TEST_USER_ID
        browser.getControl(name="form.button.Search").click()
        self.assertIn(
            '<a href="http://nohost/plone/folder1/@@remove_user_permissions?user=test_user_1_">test_user_1_</a>',
            browser.contents
        )

        # Choose user
        browser.open('http://nohost/plone/folder1/@@remove_user_permissions?user=test_user_1_')
        # Confirm link
        self.assertIn(
            '<a class="context" href="http://nohost/plone/folder1/@@remove_user_permissions?user=test_user_1_&amp;confirmed=1">',
            browser.contents
        )
        # Abort link
        self.assertIn(
            '<a class="standalone" href="http://nohost/plone/folder1/@@remove_user_permissions">',
            browser.contents
        )

        # Confirm removal of roles for the test user
        browser.open('http://nohost/plone/folder1/@@remove_user_permissions?user=test_user_1_&amp;confirmed=1')
        # After removal, redirect to permission manager overview
        self.assertIn(
            'class="template-permission_manager',
            browser.contents)

        # Approve if user has no local roles on folder1 and folder2
        self.assertFalse(
            TEST_USER_ID in dict(portal.folder1.get_local_roles()))
        self.assertFalse(
            TEST_USER_ID in dict(portal.folder1.folder2.get_local_roles()))

    def test_remove_permission_form_group(self):
        portal = self.layer['portal']
        portal_url = portal.absolute_url()
        browser = Browser(self.layer['app'])
        TEST_GROUP_ID = 'test_group'

        # Set up a group - commit change for test browser
        portal.portal_groups.addGroup(TEST_GROUP_ID)
        transaction.commit()
        # Add locale roles to a group
        portal.folder1.manage_addLocalRoles(TEST_GROUP_ID, ['Contributor',])
        portal.folder1.folder2.manage_addLocalRoles(TEST_GROUP_ID, ['Editor',])

        # Login as test user
        browser.open(portal_url + '/login_form')
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl(name='submit').click()

        browser.open(portal_url + '/folder1/remove_user_permissions')
        # Look for the right form
        self.assertIn(
            '<form method="post" action="http://nohost/plone/folder1/@@remove_user_permissions">',
            browser.contents
        )

        # Search for for the test goup
        browser.getControl(name="search_term").value = TEST_GROUP_ID
        browser.getControl(name="form.button.Search").click()
        self.assertIn(
            '<a href="http://nohost/plone/folder1/@@remove_user_permissions?user=test_group">test_group</a>',
            browser.contents
        )

        # Choose group
        browser.open('http://nohost/plone/folder1/@@remove_user_permissions?user=test_group')
        # Confirm link
        self.assertIn(
            '<a class="context" href="http://nohost/plone/folder1/@@remove_user_permissions?user=test_group&amp;confirmed=1">',
            browser.contents
        )
        # Abort link
        self.assertIn(
            '<a class="standalone" href="http://nohost/plone/folder1/@@remove_user_permissions">',
            browser.contents
        )

        # Confirm removal of roles for the test group
        browser.open('http://nohost/plone/folder1/@@remove_user_permissions?user=test_group&amp;confirmed=1')
        # After removal, redirect to permission manager overview
        self.assertIn(
            'class="template-permission_manager',
            browser.contents)

        # Approve if user has no local roles on folder1 and folder2
        self.assertFalse(
            TEST_GROUP_ID in [entry[0] for entry in portal.folder1.get_local_roles()])
        self.assertFalse(
            TEST_GROUP_ID in [entry[0] for entry in portal.folder1.folder2.get_local_roles()])



from zope.component import getUtilitiesFor, getUtility
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from plone.app.workflow.interfaces import ISharingPageRole
from plone.memoize.instance import memoize


class AdvancedSharingView(BrowserView):

    def __init__(self, *args, **kwargs):
        super(AdvancedSharingView, self).__init__(*args, **kwargs)

        self.user_selected = None
        self.user = None
        registry = getUtility(IRegistry)
        self.types = registry['ftw.permissionmanager.manage_types']

    def __call__(self, *args, **kwargs):
        self.request.set('disable_border', True)
        form = self.request.form
        self.user_selected = form.get('user', False) and True
        self.user = form.get('user', False)
        return super(AdvancedSharingView, self).__call__(*args, **kwargs)

    @memoize
    def roles(self):
        """Get a list of roles that can be managed.

        Returns a list of dics with keys:

            - id
            - title
        """
        pairs = []
        for name, utility in getUtilitiesFor(ISharingPageRole):
            pairs.append(dict(id = name, title = utility.title))

        pairs.sort(lambda x, y: cmp(x['id'], y['id']))
        return pairs

    def show_advanded_links(self):
        portal_membership = getToolByName(self.context, 'portal_membership')
        return portal_membership.checkPermission(
            'Sharing page: Delegate roles', self.context)

    @memoize
    def items(self):
        rootBrain = self.context.portal_catalog({
            'path': {
                'query': '/'.join(self.context.getPhysicalPath()),
                'depth': 0,
            },
        })[0]
        items, _index = self.brainsToItems([rootBrain])
        return items

    def brainsToItems(self, brains, index=0):
        if index > 0:
            cssClass = 'child-of-node-%i' % index
        else:
            cssClass = 'expanded'
        items = []
        contextDepth = len(self.context.getPhysicalPath())
        roleIds = [x['id'] for x in self.roles()]
        for brain in brains:
            index += 1
            # make item
            absoluteDepth = len(brain.getPath().split('/'))
            aquired = getattr(brain, 'isLocalRoleAcquired', True)
            item = {
                    'title': brain.Title,
                    'path': brain.getPath(),
                    'depth': absoluteDepth - contextDepth,
                    'cssClass': cssClass,
                    'rowid': 'node-%i' % index,
                    'isLocalRoleAcquired': aquired,
            }
            # roles
            for role in roleIds:
                item[role] = []
            if brain.get_local_roles:
                for user, roles in brain.get_local_roles:
                    if not self.user_selected or user == self.user:
                        for role in roles:
                            if role in item:
                                item[role].append(user)
            items.append(item)
            # children
            query = dict(path={
                    'query': brain.getPath(),
                    'depth': 1,
                },
                sort_on = 'getObjPositionInParent', )
            if self.types:
                query['portal_type'] = self.types
            children = self.context.portal_catalog(query)
            subItems, index = self.brainsToItems(children, index)
            items.extend(subItems)
        return items, index

    @memoize
    def users_and_groups(self):
        objectItems = self.items()
        roleIds = [x['id'] for x in self.roles()]
        userGroupMap = {}
        for item in objectItems:
            for role in roleIds:
                for user in item[role]:
                    if user not in userGroupMap:
                        userItem = self._getUserOrGroupItem(user)
                        if userItem:
                            userGroupMap[user] = userItem
                        else:
                            userGroupMap[user] = {
                                'id': user,
                                'name': user,
                                'type': 'Geloeschter Benutzer',
                            }
        # make a list
        userGroupList = userGroupMap.values()
        # sort the list by id
        userGroupList.sort(lambda a, b: cmp(a['id'], b['id']))
        return userGroupList

    def _getUserOrGroupItem(self, id_):
        item = self._getUserItem(id_)
        if item:
            return item
        item = self._getGroupItem(id_)
        if item:
            return item
        return None

    def _getUserItem(self, id_):
        user = self.context.acl_users.getUserById(id_)
        if not user:
            return None
        item = {
                'id': id_,
                'name': user.getProperty('fullname'),
                'type': 'Benutzer',
        }
        return item

    def _getGroupItem(self, id_):
        group = self.context.portal_groups.getGroupById(id_)
        if not group:
            return None
        item = {
                'id': id_,
                'name': group.getGroupTitleOrName(),
                'type': 'Gruppe',
        }
        return item



from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName


class PermissionManager(BrowserView):

    def __call__(self, *args, **kwargs):
        self.request.set('disable_border', True)
        portal_membership = getToolByName(self.context, 'portal_membership')
        # Redirect ro advanced_sharing view, if user has read only permission
        if not portal_membership.checkPermission(
            'Sharing page: Delegate roles', self.context):
            return self.context.restrictedTraverse('@@advanced-sharing')()

        return super(PermissionManager, self).__call__(*args, **kwargs)



from Products.Archetypes.utils import isFactoryContained
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.config import TOOL_NAME
from Products.CMFCore.interfaces import ICatalogTool
from Products.Archetypes.log import log
from logging import WARNING
from Acquisition import aq_base


def egov_reindexObjectSecurity(self, skip_self=False):
    if isFactoryContained(self):
        return
    at = getToolByName(self, TOOL_NAME, None)
    if at is None:
        return

    catalogs = [c for c in at.getCatalogsByType(self.meta_type)
                           if ICatalogTool.providedBy(c)]
    path = '/'.join(self.getPhysicalPath())

    for catalog in catalogs:
        for brain in catalog.unrestrictedSearchResults(path=path):
            brain_path = brain.getPath()
            if brain_path == path and skip_self:
                continue

            # Get the object
            if hasattr(aq_base(brain), '_unrestrictedGetObject'):
                ob = brain._unrestrictedGetObject()
            else:
                # BBB: Zope 2.7
                ob = self.unrestrictedTraverse(brain_path, None)
            if ob is None:
                # BBB: Ignore old references to deleted objects.
                # Can happen only in Zope 2.7, or when using
                # catalog-getObject-raises off in Zope 2.8
                log("reindexObjectSecurity: Cannot get %s from catalog" %
                    brain_path, level=WARNING)
                continue
            # Also append our new index
            indexes = list(self._cmf_security_indexes)
            indexes.append('get_local_roles')
            indexes.append('isLocalRoleAcquired')
            # Recatalog with the same catalog uid.
            catalog.reindexObject(ob, idxs=tuple(indexes),
                                    update_metadata=1, uid=brain_path)



from plone.indexer import indexer
from Products.Archetypes.interfaces import IBaseObject


@indexer(IBaseObject)
def global_get_local_roles(object):
    return object.get_local_roles()


@indexer(IBaseObject)
def isLocalRoleAcquired(object):
    if getattr(object, '__ac_local_roles_block__', None):
        return False
    return True



from zope.interface import implements
from zope.component import getMultiAdapter

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from kss.core.interfaces import IKSSView
from plone.app.kss.plonekssview import PloneKSSView as base


class KSSSharingView(base):
    """KSS view for sharing page.
    """
    implements(IKSSView)

    template = ViewPageTemplateFile('sharing.pt')
    macro_wrapper = ViewPageTemplateFile('macro_wrapper.pt')

    def updateSharingInfo(self, search_term=''):
        sharing = getMultiAdapter(
            (self.context, self.request),
            name="sharing")

        # get the html from a macro
        ksscore = self.getCommandSet('core')

        the_id = 'user-group-sharing'
        macro = self.template.macros[the_id]
        res = self.macro_wrapper(
            the_macro=macro,
            instance=self.context,
            view=sharing)
        ksscore.replaceHTML(ksscore.getHtmlIdSelector(the_id), res)

        return self.render()



from setuptools import setup, find_packages
import os

version = '1.4.5'
maintainer = 'Mathias Leimgruber'

tests_require = ['plone.app.testing', ]

setup(name='ftw.permissionmanager',
      version=version,
      description="",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.0',
        'Framework :: Plone :: 4.1',
        "Programming Language :: Python",
        ],
      keywords='',
      author='%s, 4teamwork GmbH' % maintainer,
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='http://github.com/4teamwork/ftw.permissionmanager',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      install_requires=[
          'setuptools',
          'collective.monkeypatcher',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.testing import z2
from zope.configuration import xmlconfig
from plone.app.testing import setRoles, TEST_USER_ID, TEST_USER_NAME, login
import ftw.permissionmanager


TEST_USER_ID_2 = '_test_user_2_'
TEST_USER_PW_2 = 'secret'
TEST_GROUP_ID = 'test_group'
TEST_GROUP_ID_2 = 'test_group_2'


class FtwPermissionmanagerLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        # Load ZCML

        xmlconfig.file('configure.zcml',
            ftw.permissionmanager,
            context=configurationContext)

        # installProduct() is *only* necessary for packages outside
        # the Products.* namespace which are also declared as Zope 2 products,
        # using <five:registerPackage /> in ZCML.
        z2.installProduct(app, 'ftw.permissionmanager')

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.permissionmanager:default')

        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        # Create one folder with one item
        portal.invokeFactory('Folder', 'folder1', title='Folder1')
        portal.folder1.invokeFactory('Folder', 'folder2', title='Folder2')

        # Add a new user
        portal.portal_registration.addMember(TEST_USER_ID_2, TEST_USER_PW_2)


FTW_PERMISSIONMANAGER_FIXTURE = FtwPermissionmanagerLayer()
FTW_PERMISSIONMANAGER_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FTW_PERMISSIONMANAGER_FIXTURE, ),
    name="ftw.permissionmanager:Integration")



from zope.i18nmessageid import MessageFactory
import csv


permission_manager_factory = MessageFactory('ftw.permissionmanager')


class excel_ger(csv.excel):
    """Extend csv library with custem dialect excel_ger"""
    delimiter = ';'
csv.register_dialect('excel_ger', excel_ger)


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



from Acquisition import aq_inner
from plone.app.workflow.browser.sharing import SharingView as base
from plone.app.workflow.interfaces import ISharingPageRole
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode, getSiteEncoding
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtilitiesFor


class SharingView(base):
    """This sharing view works a bit diffrent than the one from plone
       The search result is displayed in his own table.
       If a role is selected in search result table, the entry will be added
       to current_settings, this (kind of) saves the result for the user
    """

    template = ViewPageTemplateFile('sharing.pt')

    def has_manage_portal(self):
        return self.context.portal_membership.checkPermission(
            'ManagePortal',
            self.context)

    def has_local_role(self, role=''):
        local_roles = self.context.aq_explicit.get_local_roles()
        authenticated_member = \
            self.context.portal_membership.getAuthenticatedMember().id
        for userid, roles in local_roles:
            if (authenticated_member == userid and role in roles) \
                or self.has_manage_portal():
                return True
        return False

    def role_settings(self):
        """Only return current settings"""
        results = self.existing_role_settings()
        encoding = getSiteEncoding(aq_inner(self.context))
        static = {}
        index = None
        # Puts AuthenticatedUsers group to position 0
        for item in results:
            if item['id'] == 'AuthenticatedUsers':
                index = results.index(item)
                break
        if index != -1 and index is not None:
            static = results.pop(index)
        results.sort(
            lambda x, y: cmp(
                safe_unicode(x["title"], encoding).lower(),
                safe_unicode(y["title"], encoding).lower()))
        # Only show AuthenticatedUsers group users with ManagePortal
        # perission
        # XXX: This should be configurable
        if static and self.has_manage_portal():
            results.insert(0, static)
        return results

    def search_result(self):
        """Returns only the search result for the new search result table"""

        # XXX: This method is inefficient. To many interations and
        # manipulations. # If someone really understands what to do,
        # please do it.

        mtool = getToolByName(self.context, 'portal_membership')
        gtool = getToolByName(self.context, 'portal_groups')

        user_results = self.user_search_results()
        group_results = self.group_search_results()
        current_settings = user_results + group_results

        requested = self.request.form.get('entries', [])

        # Remove real/saved roles from requested
        for item in self.existing_role_settings():
            index = None
            for r_item in requested:
                if item['id'] == r_item['id']:
                    index = requested.index(r_item)
                    break
            if index is not None:
                requested.pop(index)

        if requested is not None:
            knownroles = [r['id'] for r in self.roles()]
            settings = {}
            result_like_settings = {}
            for entry in requested:
                roles = []
                roles_for_settings = {}
                for r in knownroles:
                    if entry.get('role_%s' % r, False):
                        roles.append(r)
                        roles_for_settings[r] = True
                    else:
                        roles_for_settings[r] = False
                    settings[(entry['id'], entry['type'])] = roles
                # Add requested entries to current_settings if there is one or
                #  more roles selected
                # This is kind a temporary storage for search results.
                # It allows you to do multible search queries and you will not
                # lose allready selected roles for user/groups
                if roles and not self.request.get('form.button.Save', None):

                    # get group title or user fullname
                    if entry['type'] == 'user':
                        member = mtool.getMemberById(entry['id'])
                        title = '%s' % (
                            member.getProperty('fullname', entry['id']))
                    else:
                        group = gtool.getGroupById(entry['id'])
                        title = group.getGroupTitleOrName()
                    result_like_settings = {
                        'id': entry['id'],
                        'type': entry['type'],
                        'title': title,
                        'roles': roles_for_settings}

                    # Add or update settings
                    updated = False
                    for given_entry in current_settings:
                        if result_like_settings['id'] == given_entry['id']:
                            given_entry.update(result_like_settings)
                            updated = True
                            break
                    if not updated:
                        current_settings.append(result_like_settings)
            for entry in current_settings:
                desired_roles = settings.get(
                    (entry['id'], entry['type']),
                    None)

                if desired_roles is None:
                    continue
                for role in entry["roles"]:
                    if entry["roles"][role] in [True, False]:
                        entry["roles"][role] = role in desired_roles

        encoding = getSiteEncoding(aq_inner(self.context))
        current_settings.sort(
            lambda x, y: cmp(
                safe_unicode(x["title"], encoding).lower(),
                safe_unicode(y["title"], encoding).lower()))

        return current_settings

    def roles(self):
        """Get a list of roles that can be managed.

        Returns a list of dicts with keys:

            - id
            - title
        """
        context = aq_inner(self.context)

        pairs = []
        has_manage_portal = context.portal_membership.checkPermission(
            'ManagePortal',
            context)
        aviable_roles_for_users = [
            u'Editor',
            u'Reader',
            u'Contributor',
            u'Administrator']
        for name, utility in getUtilitiesFor(ISharingPageRole):
            if not has_manage_portal and name not in aviable_roles_for_users:
                continue
            pairs.append(dict(id=name, title=utility.title))

        pairs.sort(key=lambda x: x["id"])
        return pairs
